% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffusionmap-methods.r
\name{DiffusionMap methods}
\alias{DiffusionMap methods}
\alias{print.DiffusionMap}
\alias{show.DiffusionMap}
\alias{print,DiffusionMap-method}
\alias{show,DiffusionMap-method}
\title{DiffusionMap methods}
\usage{
\S4method{print}{DiffusionMap}(x)

\S4method{show}{DiffusionMap}(object)
}
\arguments{
\item{x, object}{A \code{\link{DiffusionMap}}}
}
\value{
The \code{DiffusionMap} object (\code{print}), or NULL (\code{show}), invisibly
}
\description{
Methods for external operations on diffusion maps
}
\examples{
data(guo)
dm <- DiffusionMap(guo)
print(dm)
show(dm)

}
\seealso{
\link{DiffusionMap accession methods}, \link{Extraction methods}, \link{Coercion methods} for more
}
